//=============================================================================
// ANNADeathMatch. (C) 2012 "Dei" & Nobody
//=============================================================================
/**
  Gametype for the Deathmatch Game.
*/
class ANNADeathMatch extends MTLDeathMatch;

var ANNAManager Manager;

const BOOL_FALSE = 0;
const BOOL_TRUE = 1;

/**
  Sets the MPL depending on the just loaded map.
*/
event InitGame(string Options, out string Error)
{
    local string LogFile;

    LogFile = ParseOption(Options, "Log");
    if(Len(LogFile) > 0)
        ServerLogName = LogFile;

    Super.InitGame(Options, Error);

    Manager = findManager();

    CBPMutator(level.Game.BaseMutator).AddCBPMutator(Spawn(class'_ANNAMutator'));
}

function PostPostBeginPlay()
{
    local string newScoreBoard, newGameName;
    local class<EMMAScoreBoard> newScoreBoardClass;

    Super.PostPostBeginPlay();

    if(Manager == None)
        Manager = findManager();

    newScoreBoard = Manager.ScoreBoard();

    if(Len(newScoreBoard) > 0)
    {
        newScoreBoardClass = class<EMMAScoreBoard>(DynamicLoadObject(newScoreBoard, class'Class'));
        if(newScoreBoardClass != None)
        {
            ScoreboardType = newScoreBoardClass;
        }
    }

    Manager.MenuMain();

    newGameName = Manager.GameName();

    if(Len(newGameName) > 0)
    {
        GameName = newGameName;
    }
}

/**
  Find and return the ANNAManager
*/
function ANNAManager findManager()
{
    local ANNAManager curManager;
    local bool bFound;

    if(Manager != None)
        return Manager;

    foreach AllActors(class'ANNAManager', curManager)
        if(curManager != None)
            return curManager;
            
    curManager = Spawn(class'ANNAManager'); // We didn't find any Manager class, so we spawn one!
    return curManager;
}

event PlayerPawn Login (string Portal, string URL, out string Error, Class<PlayerPawn> SpawnClass)
{
    local MTLPlayer newPlayer;
    local string classStr, purgedClassStr;
    local int pkgLength;
    local Pawn PawnLink;
    local PlayerPawn TestPlayer;

    if((MaxPlayers > 0) && (NumPlayers >= MaxPlayers) )
    {
        Error = TooManyPlayers;
        return None;
    }

    for(PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.NextPawn)
    {
        TestPlayer = PlayerPawn(PawnLink);
        if(TestPlayer != None)
        {
            if(Len(TestPlayer.Password) == 0)
            {
                TestPlayer.Password = class'ANNAManager'.static.generateUID();
            }
        }
    }

    SpawnClass = DefaultPlayerClass;
    classStr = ParseOption(URL,"Class");
    pkgLength = InStr(classStr,".");
    if(pkgLength != -1 )
    {
        purgedClassStr = Mid(classStr,pkgLength + 1);
        classStr = Left(classStr,pkgLength);
    }
    else
    {
        purgedClassStr = classStr;
        classStr="";
    }
    if((purgedClassStr ~= "MPNSF") || (purgedClassStr ~= "MTLNSF") )
        SpawnClass = class'ANNANSF';
    else if((purgedClassStr ~= "MPUNATCO") || (purgedClassStr ~= "MTLUNATCO") )
        SpawnClass = class'ANNAUNATCO';
    else if((purgedClassStr ~= "MPMJ12") || (purgedClassStr ~= "MTLMJ12") )
        SpawnClass = class'ANNAMJ12';

    ChangeOption(URL,"Class",string(SpawnClass));
        newPlayer = MTLPlayer(super(DeathMatchGame).Login(Portal,URL,Error,SpawnClass));
    if(newPlayer != None)
        newPlayer.V52(newPlayer.PlayerReplicationInfo.PlayerName);

    return newPlayer;
}

exec function ConsoleKick(int playerID)
{
    if(Manager == None)
        Manager = findManager();

    Manager.serverKick(playerID);
}

exec function ConsoleKickBan(int playerID)
{
    if(Manager == None)
        Manager = findManager();

    Manager.serverBan(playerID);
}

exec function ConsolePlayerList()
{
    if(Manager == None)
        Manager = findManager();

    Manager.serverPlayerList();
}

event PostLogin(playerpawn NewPlayer)
{
    local _ANNAPlayer DXPlayer;

    Super.PostLogin(NewPlayer);

    DXPlayer = _ANNAPlayer(NewPlayer);
    if(DXPlayer != None)
        DXPlayer.CheckInventory();
}

/**
  Disable haxy hax.
*/
function EditActor();

/**
  Module related functions
*/
function ScoreKill(pawn Killer, pawn Other)
{
    if(Manager == None)
        Manager = findManager();

    Manager.ScoreKill(Killer, Other);
}

function Logout(pawn Exiting)
{
    if(Manager == None)
        Manager = findManager();

    Manager.Logout(Exiting);

	Super.Logout(Exiting);
}

function NavigationPoint FindPlayerStart(Pawn Player, optional byte InTeam, optional string incomingName)
{
    local NavigationPoint moduleNavigationPoint;
    if(Manager == None)
        Manager = findManager();

    moduleNavigationPoint = Manager.FindPlayerStart(Player, InTeam, incomingName);
    if(moduleNavigationPoint != None)
        return moduleNavigationPoint;

    return Super.FindPlayerStart(Player, InTeam, incomingName);
}

event PreLogin(string Options, string Address, out string Error, out string FailCode)
{
    local int bOverride;
    local string moduleError, moduleFailCode;

    if(Manager == None)
        Manager = findManager();

    Manager.PreLogin(Options, Address, moduleError, moduleFailCode, bOverride);

    if(Len(moduleError) != 0 || bOverride == BOOL_TRUE)
    {
        Error = moduleError;
        FailCode = moduleFailCode;
        return;
    }

    Super.PreLogin(Options, Address, Error, FailCode);
}

function HandleDeathNotification(Pawn killer, Pawn killee)
{
    local int bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.HandleDeathNotification(killer, killee, bOverride);

    if(bOverride != BOOL_TRUE)
        Super.HandleDeathNotification(killer, killee);
}

function Killed(pawn Killer, pawn Other, name damageType)
{
    local int bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.Killed(Killer, Other, damageType, bOverride);
    if(bOverride != BOOL_TRUE)
        Super.Killed(Killer, Other, damageType);
}

function bool CheckVictoryConditions(Pawn Killer, Pawn Killee, String Method)
{
    local bool moduleVictory;
    local int bOverride;

    if(Manager == None)
        Manager = findManager();

    moduleVictory = Manager.CheckVictoryConditions(Killer, Killee, Method, bOverride);
    if(bOverride == BOOL_TRUE)
        return moduleVictory;

    return Super.CheckVictoryConditions(Killer, Killee, Method);
}

function bool ChangeTeam(Pawn PawnToChange, int NewTeam)
{
    local int bOverride;
    local int CurTeam;
    local bool bResult;

    if(PlayerPawn(PawnToChange) != None)
        CurTeam = PlayerPawn(PawnToChange).PlayerReplicationInfo.Team;

    if(Manager == None)
        Manager = findManager();

    Manager.ChangeTeam(PawnToChange, NewTeam, bOverride);
    if(bOverride != BOOL_TRUE)
    {
        bResult = Super.ChangeTeam(PawnToChange, NewTeam);
        if(bResult)
        {
            if(_ANNAPlayer(PawnToChange) != None)
            {
                _ANNAPlayer(PawnToChange).clientChangeTeam(NewTeam);
                return true;
            }
        }
        else
            return false;
    }
    else
    {
        if(CurTeam != NewTeam)
        {
            if(_ANNAPlayer(PawnToChange) != None)
            {
                _ANNAPlayer(PawnToChange).clientChangeTeam(CurTeam);
            }
        }
        return false;
    }
}

DefaultProperties
{
    GameName="Deathmatch"
    DefaultPlayerClass=class'ANNAJCDenton'
    ScoreboardType=Class'ANNAScoreboard'
}
