//=============================================================================
// ANNAManager. (C) 2012 "Dei" & Nobody
//=============================================================================
class ANNAManager extends Actor config(ANNA);

var() config float MaxFrobDistance;
var() config bool bEnableSpectating;
var() config bool bResetSkin;
var() config bool bRotateLogs; // Set to false by default; needs to be tested properly in combination with automatic restart scripts
var() config string LogDirectory;
var() config string DefaultMap;
//var() config bool bDrainEnergyOnUsage;

var EMMAManager EMMAManager;
var string customMenuMain;

const ANNA_VERSION = "103"; // Current version number. It's changed by every update.

function BeginPlay()
{
    local ANNAManager curManager;
    local bool bFound;
    local string currentMap;
    local class<GameInfo> currentGameClass, newGameClass;
    local int testint;

    foreach AllActors(class'ANNAManager', curManager)
    if(curManager != None && curManager != self)
        bFound = true;

    if(!bFound && ROLE == ROLE_Authority)
    {
        currentGameClass = Level.Game.Class;
        currentMap = left(string(Level), instr(string(Level), "."));
        switch(currentGameClass)
        {
            case class'DeathMatchGame':   newGameClass = class'ANNADeathMatch'; break;
            case class'TeamDMGame':       newGameClass = class'ANNATeam';       break;
            case class'BasicTeamDMGame':  newGameClass = class'ANNABasicTeam';  break;
            case class'AdvTeamDMGame':    newGameClass = class'ANNAAdvTeam';    break;
            case class'MTLDeathMatch':    newGameClass = class'ANNADeathMatch'; break;
            case class'MTLTeam':          newGameClass = class'ANNATeam';       break;
            case class'MTLBasicTeam':     newGameClass = class'ANNABasicTeam';  break;
            case class'MTLAdvTeam':       newGameClass = class'ANNAAdvTeam';    break;
        }

        if(currentMap == "DX" && Len(DefaultMap) > 0)
            currentMap = DefaultMap;

        if(newGameClass != None)
        {
            Log("Loading ANNA...", 'ANNA');

            if(bRotateLogs)
            {
                if(Len(LogDirectory) > 0)
                    ConsoleCommand("relaunch"@currentMap$"?Game="$string(newGameClass)$"?-server?log="$LogDirectory$"\\"$Level.Year$"-"$Level.Month$"-"$Level.Day$"_"$Level.Hour$"-"$Level.Minute$"-"$Level.Second$".ANNA.server.log");
                else
                    ConsoleCommand("relaunch"@currentMap$"?Game="$string(newGameClass)$"?-server?log="$Level.Year$"-"$Level.Month$"-"$Level.Day$"_"$Level.Hour$"-"$Level.Minute$"-"$Level.Second$".ANNA.server.log");
            }
            else
                ConsoleCommand("servertravel"@currentMap$"?Game="$string(newGameClass));
        }
        else
        {
            if(bRotateLogs && inStr(Level.Game.ServerLogName, ".ANNA.") == -1)
            {
                if(Len(LogDirectory) > 0)
                    ConsoleCommand("relaunch"@currentMap$"?Game="$string(newGameClass)$"?-server?log="$LogDirectory$"\\"$Level.Year$"-"$Level.Month$"-"$Level.Day$"_"$Level.Hour$"-"$Level.Minute$"-"$Level.Second$".ANNA.server.log");
                else
                    ConsoleCommand("relaunch"@currentMap$"?Game="$string(newGameClass)$"?-server?log="$Level.Year$"-"$Level.Month$"-"$Level.Day$"_"$Level.Hour$"-"$Level.Minute$"-"$Level.Second$".ANNA.server.log");
            }
            else
            {
                EMMAManager = Spawn(class'EMMAManager');
            }
        }
    }
    else
        Destroy();

    Super.BeginPlay();
}

/**
 Server console kick.
*/
function serverKick(int playerID)
{
    local Pawn P;
    local string playerName;

    for(P=Level.PawnList; P!=None; P=P.NextPawn)
    {
        if(_ANNAPlayer(P) != None && P.PlayerReplicationInfo != None)
        {
            if(P.PlayerReplicationInfo.PlayerID == playerID)
            {
                playerName = P.PlayerReplicationInfo.PlayerName@"("$P.PlayerReplicationInfo.PlayerID$")";
                _ANNAPlayer(P).V7D = true;
                P.Destroy();
                Log(playername@"has been kicked.", 'ANNA');
                return;
            }
        }
    }
    Log("Could not find a player with ID"@playerID, 'ANNA');
}

/**
 Server console ban.
*/
function serverBan(int playerID) 
{
    local Pawn P;
    local string IP, playerName;
    local int i;

    for(P=Level.PawnList; P!=None; P=P.NextPawn)
    {
        if(_ANNAPlayer(P) != None && P.PlayerReplicationInfo != None && NetConnection(PlayerPawn(P).Player) != None)
        {
            if(P.PlayerReplicationInfo.PlayerID == playerID)
            {
                IP = _ANNAPlayer(P).GetPlayerNetworkAddress();
                if(Level.Game.CheckIPPolicy(IP))
                {
                    playerName = P.PlayerReplicationInfo.PlayerName@"("$P.PlayerReplicationInfo.PlayerID$")";
                    IP = Left(IP, InStr(IP, ":"));
                    Log("Adding IP Ban for: "$IP, 'ANNA');
                    for(i=0; i<50; i++)
                        if(Level.Game.IPPolicies[i] == "")
                            break;

                    if(i < 50)
                        Level.Game.IPPolicies[i] = "DENY,"$IP;

                    Level.Game.SaveConfig();
                }
                P.Destroy();
                Log(playername@"has been banned & kicked.", 'ANNA');
                return;
            }
        }
    }
    Log("Could not find a player with ID"@playerID, 'ANNA');
}

function serverPlayerList()
{
    local Pawn P;
    local string IP;

    Log("	ID		PLAYERNAME		IP-ADDRESS", 'ANNA');
    for(P=Level.PawnList; P!=None; P=P.NextPawn)
    {
        if(_ANNAPlayer(P) != None && P.PlayerReplicationInfo != None)
        {
            IP = _ANNAPlayer(P).GetPlayerNetworkAddress();
            IP = Left(IP, InStr(IP, ":"));
            Log("	"$P.PlayerReplicationInfo.PlayerID$"		"$P.PlayerReplicationInfo.PlayerName$"		"$IP, 'ANNA');
        }
    }
}

/**
 Generates a 16 byte long Hexa-Decimal Unique ID.
*/
static function string generateUID()
{
    local int i;
    local string UID;

    for(i=0; i<16; i++)
    {
        if(FRand() < 0.5)
            UID = UID$string(Rand(9));
        else
            UID = UID$Chr(65+Rand(5));
    }
    return UID;
}

/**
  Module related functions
*/
final function ScoreKill(pawn Killer, pawn Other)
{
    if(EMMAManager != None)
        EMMAManager.ScoreKill(Killer, Other);
}

final function PlayerDestroyed(DeusExPlayer Player)
{
    if(EMMAManager != None)
        EMMAManager.PlayerDestroyed(Player);
}

final function Logout(pawn Exiting)
{
    if(EMMAManager != None)
        EMMAManager.Logout(Exiting);
}

final function PlayerPossess(DeusExPlayer Player)
{
    if(EMMAManager != None)
        EMMAManager.PlayerPossess(Player);
}

final function NavigationPoint FindPlayerStart(Pawn Player, optional byte InTeam, optional string incomingName)
{
    if(EMMAManager != None)
        return EMMAManager.FindPlayerStart(Player, InTeam, incomingName);

    return None;
}

final function PreLogin(string Options, string Address, out string Error, out string FailCode, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.PreLogin(Options, Address, Error, FailCode, bOverride);
}

final function HandleDeathNotification(Pawn killer, Pawn killee, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.HandleDeathNotification(killer, killee, bOverride);
}

final function Killed(pawn Killer, pawn Other, name damageType, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.Killed(Killer, Other, damageType, bOverride);
}

final function bool CheckVictoryConditions(Pawn Killer, Pawn Killee, String Method, out int bOverride)
{
    if(EMMAManager != None)
        return EMMAManager.CheckVictoryConditions(Killer, Killee, Method, bOverride);

    return false;
}

final function ChangeTeam(Pawn PawnToChange, out int NewTeam, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.ChangeTeam(PawnToChange, NewTeam, bOverride);
}

final function string ScoreBoard()
{
    if(EMMAManager != None)
        return EMMAManager.ScoreBoard();

    return "";
}

final function MenuMain()
{
    if(EMMAManager != None)
        customMenuMain = EMMAManager.MenuMain();
}

final function GiveInitialInventory(DeusExPlayer Player, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.GiveInitialInventory(Player, bOverride);
}

final function string GameName()
{
    if(EMMAManager != None)
        return EMMAManager.GameName();

    return "";
}

final function SetName(DeusExPlayer Player, string NewName, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.SetName(Player, NewName, bOverride);
}

final function PlayerCommand(DeusExPlayer Player, string Command, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.PlayerCommand(Player, Command, bOverride);
}

final function DropItem(DeusExPlayer Player, Inventory Item, out int bOverride)
{
    if(EMMAManager != None)
        EMMAManager.DropItem(Player, Item, bOverride);
}

defaultproperties
{
    MaxFrobDistance=112.00
    bEnableSpectating=true
    bResetSkin=true
    bRotateLogs=false
    bHidden=true
}
