//=============================================================================
// _ANNAAntiCheat. (C) 2012 "Dei" & Nobody
//=============================================================================
/**
 Checks the consoleclass of a player.
 It's spawned by the engine after a player joined to check his/her client-initalized consoleclass.
 The check algorithm is nearly unbreakable because the server just requests
 the consoleclass information from the client and checks it server-side. (like miniMTL143 did it)
 However the difference to miniMTL143's check is, that miniMTL210 starts a timeout for the client to answer.
 So if some haxor hacked his copy of miniMTL210 to avert, that it sends his consoleclass information to the server,
 he will be kicked by the timeout.
 Also this check algorithm comes with a UID which is dynamically generated for the check session,
 so the haxor-client can't send a valid fake answer.

*/
class _ANNAAntiCheat extends Actor;

replication
{
    reliable if(ROLE == ROLE_Authority)
        _clientGiveInfo;

    reliable if(ROLE < ROLE_Authority)
        _serverValidateInfo;
}

/** Saves the UID which is generated for the console-class check to validate the answer of the client. */
var private string _checkUID;
var private string _foundConsole;

/** @ignore */
var private bool _bCheckPassed;
var private bool _bCheatDetected;
var private bool _bInitalized;

var string _rootWindowList[3];

/**
 Starts a Timer of 5 seconds (to let the server time to send the just joined client all game relevant information)
 After that 5 seconds <i>Initalize()</i> function will be called to start the console check.
*/
function PostBeginPlay()
{
    if(_ANNAPlayer(Owner) != None)
    {
        if(level.NetMode != NM_StandAlone && ROLE == ROLE_Authority)
        {
            SetTimer(4, false);
        }
    }
}

/**
 Generating the UID, sending a check request to the client and starting the TimeOut
*/
function _Initalize()
{
    if(level.NetMode != NM_StandAlone && ROLE == ROLE_Authority)
    {
        if(_ANNAPlayer(Owner) != none)
        {
            _checkUID = class'ANNAManager'.static.generateUID();
            _clientGiveInfo(_checkUID, "Engine.Engine", "Engine.GameEngine", "Console", "GameEngine", "Root");
            _bInitalized = True;
            SetTimer(4, false);
        }
    }
}

/**
 Sends a check request to the client, which will return the console class over <i>serverValidateConsole()</i>.
*/
simulated function _clientGiveInfo(string _checkID, string _EngineSection, string _GameEngineSection, string _Console, string _GameEngine, string _Root)
{
    _serverValidateInfo(_ANNAPlayer(Owner).Player.Console.Class, string(_ANNAPlayer(Owner).Player.Console.Class), GetConfig(_EngineSection, _Console), GetConfig(_GameEngineSection, _Console), GetConfig(_EngineSection, _GameEngine), _ANNAPlayer(Owner).rootWindow.Class, string(_ANNAPlayer(Owner).rootWindow.Class), GetConfig(_EngineSection, _Root), _checkID);
}

/**
 Sends the answer with the requested information back to the server.
*/
function _serverValidateInfo(Class<Console> _consoleClass, string _consoleClassString, string _consoleString1, string _consoleString2, string _gameEngine, Class<RootWindow> _rootClass, string _rootClassString, string _rootString, string _checkID)
{
    local bool _bConsolePassed, _bEnginePassed, _bRootPassed;
    local int _i;
    local class <RootWindow> _RootWindow;

    if(_checkID == _checkUID)
    {
        if(_consoleClass == Class'Engine.Console' && _consoleClassString == "Engine.Console")
        {
            if((Len(_consoleString1) > 0 && _consoleString1 == "Engine.Console") || Len(_consoleString1) == 0)
            {
                if((Len(_consoleString2) > 0 && _consoleString2 == "Class'Engine.Console'") || Len(_consoleString2) == 0)
                {
                    _bConsolePassed = true;
                }
            }
        }

        if(!_bConsolePassed)
            _reportHack("Invalid console", _consoleClass$"|"$_consoleClassString$"|"$_consoleString1$"|"$_consoleString2);

        if(Len(_gameEngine) > 0 && _gameEngine == "DeusEx.DeusExGameEngine")
            _bEnginePassed = true;
        else
            _reportHack("Invalid engine", _gameEngine);

        _RootWindow = class<RootWindow>(DynamicLoadObject("DeusEx.DeusExRootWindow", class'Class'));
        if(_RootWindow != None && _rootClass == _RootWindow && string(_rootClass) == _rootClassString)
        {
            if((Len(_rootString) > 0 && _rootString == "DeusEx.DeusExRootWindow") || Len(_rootString) == 0)
            {
                _bRootPassed = true;
            }
        }

        if(!_bRootPassed)
        {
            for(_i = 0; _i < 3; _i++)
            {
                if(Len(_rootWindowList[_i]) > 0)
                {
                    _RootWindow = class<RootWindow>(DynamicLoadObject(_rootWindowList[_i], class'Class'));
                    if(_RootWindow != None && _rootClass == _RootWindow && string(_rootClass) == _rootClassString)
                    {
                        if((Len(_rootString) > 0 && _rootString == _rootWindowList[_i]) || Len(_rootString) == 0)
                        {
                            _bRootPassed = true;
                            break;
                        }
                    }
                }
            }
        }

        if(!_bRootPassed)
            _reportHack("Invalid rootwindow", _rootString);

        if(_bConsolePassed && _bEnginePassed && _bRootPassed)
            _bCheckPassed = true;
    }
}

function _reportHack(string _cheatMethod, string _cheatDetails)
{
    if(DeusExPlayer(Owner) != None)
    {
        Log(" ");
        Log("===============================");
        Log("         CHEAT FOUND           ");
        Log("-------------------------------");
        Log("Player Caught:   " $ DeusExPlayer(Owner).PlayerReplicationInfo.PlayerName $ ".");
        Log("Player Address:  " $ DeusExPlayer(Owner).GetPlayerNetworkAddress() $ ".");
        Log("Date:            " $ string(Level.Month) $ "/" $ string(Level.Day) $ "/" $ string(Level.Year) $ ".");
        Log("Time:            " $ string(Level.Hour) $ ":" $ string(Level.Minute) $ ":" $ string(Level.Second) $ ".");
        Log("Cheat Found:     " $ _cheatMethod $ ".");
        Log("Cheat Details:   " $ _cheatDetails $ ".");
        Log("===============================");
        Log(" ");
        DeusExPlayer(Owner).ForceDisconnect(_cheatMethod$", disconnecting");
        DeusExPlayer(Owner).Destroy(); // Just in case..
    }
}

/**
 Is called by the engine, when the timeout has exceeded.
 if the server didn't get a valid answer yet, it will disconnect the client.
*/
simulated function Timer()
{
    if(DeusExPlayer(Owner) != None)
    {
        if(_bInitalized)
        {
            if(!_bCheckPassed)
            {
                DeusExPlayer(Owner).ForceDisconnect("Failed running AntiCheat, disconnecting");
                DeusExPlayer(Owner).Destroy(); // Just in case..
            }
            else
            {
                _bInitalized = false;
                SetTimer(90+Rand(30), false);
            }
        }
        else
        {
            _Initalize();
        }
    }
    else
        Destroy();
}

DefaultProperties
{
    bHidden=true
    _rootWindowList(0)="DeusEx.DeusExRootWindow"
}
