//=============================================================================
// _ANNAAugmentationManager. (C) 2012 "Dei" & Nobody
//=============================================================================
class _ANNAAugmentationManager extends CBPAugmentationManager;


/**
  Check if the player is allowed to receive an augmentation.
*/
function Augmentation GivePlayerAugmentation(Class<Augmentation> giveClass)
{
	local Augmentation anAug;
    
    if(bNetOwner) // Should protect it from being called clientside
        return None;
        
    //Log("We are meant to give the player an"@giveClass);
        
    giveClass = ReplaceAug(giveClass);
    //Log("So we're going to give him"@giveClass);
        
	// Checks to see if the player already has it.  If so, we want to 
	// increase the level
	anAug = FindAugmentation(giveClass);
    //Log(anAug@anAug.bHasIt);
        
	if (anAug == None)
		return None;		// shouldn't happen, but you never know!

	if (anAug.bHasIt)
	{
		anAug.IncLevel();
		return anAug;
	}

	if (AreSlotsFull(anAug))
	{
		Player.ClientMessage(AugLocationFull);
		return anAug;
	}

	anAug.bHasIt = True;

	if (anAug.bAlwaysActive) //  || specialAug(anAug))
	{
		anAug.bIsActive = True;
		anAug.GotoState('Active');
	}
	else
	{
		anAug.bIsActive = False;
	}


	if ( Player.Level.Netmode == NM_Standalone )
		Player.ClientMessage(Sprintf(anAug.AugNowHaveAtLevel, anAug.AugmentationName, anAug.CurrentLevel + 1));

	// Manage our AugLocs[] array
	AugLocs[anAug.AugmentationLocation].augCount++;
	
	// Assign hot key to new aug 
	// (must be after before augCount is incremented!)
   if (Level.NetMode == NM_Standalone)	
      anAug.HotKeyNum = AugLocs[anAug.AugmentationLocation].augCount + AugLocs[anAug.AugmentationLocation].KeyBase;
   else
      anAug.HotKeyNum = anAug.MPConflictSlot + 2;

	if ((!anAug.bAlwaysActive) && (Player.bHUDShowAllAugs))
    {
	    Player.AddAugmentationDisplay(anAug);
    }

	return anAug;
}

simulated function Augmentation FindAugmentation(Class<Augmentation> findClass)
{
	local Augmentation anAug, currentAug;

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;

        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);
        
		if(currentAug.Class == findClass)
        {
            anAug = currentAug;
			break;
        }

		anAug = anAug.next;
	}

	return anAug;
}

simulated function Float CalcEnergyUse(float deltaTime)
{
    local float energyUse, energyMult, energyRate;
    local Augmentation anAug, currentAug;
    local Augmentation PowerAug;
    local bool bActiveAug;

	energyUse = 0;
	energyMult = 1.0;

	anAug = FirstAug;
    while(anAug != None)
    {
        currentAug = anAug;
        
        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);
        
        if(currentAug.IsA('AugPower'))
            PowerAug = currentAug;

        if(currentAug.bHasIt && currentAug.bIsActive)
        {
            energyRate = currentAug.GetEnergyRate();
            if(energyRate > 0)
                bActiveAug = true;

            /*if(!_ANNAPlayer(Player).Manager.bDrainEnergyOnUsage || (currentAug.IsA('AugSpeed') && VSize(Player.Velocity) > 10 && Player.Physics == PHYS_Walking) || (currentAug.IsA('AugAqualung') && Player.IsInState('PlayerSwimming')) || !specialAug(currentAug))*/
                energyUse += ((energyRate/60) * deltaTime);

            if(currentAug.IsA('AugPower'))
                energyMult = currentAug.LevelValues[currentAug.CurrentLevel];
        }
        anAug = anAug.next;
    }

    if((Level.NetMode != NM_Standalone) && (PowerAug != None) && (PowerAug.bHasIt))
    {
        if(bActiveAug && !PowerAug.bIsActive)
            ActivateAugByKey(PowerAug.HotKeyNum - 3);

        if(!bActiveAug && PowerAug.bIsActive)
            ActivateAugByKey(PowerAug.HotKeyNum - 3);

        if(PowerAug.bIsActive)				
            energyMult = PowerAug.LevelValues[PowerAug.CurrentLevel];
    }

	energyUse *= energyMult;

	return energyUse;
}

/**
  Should the player be spectating, pretend he has the Radar Transparency augmentation.
*/
simulated function int GetClassLevel(class<Augmentation> augClass)
{
	local Augmentation anAug, currentAug;
    
    if(Player != None && Player.PlayerReplicationInfo != None)
        if(Player.PlayerReplicationInfo.bIsSpectator)
            if(augClass == Class'DeusEx.AugRadarTrans')
                return 3;
                
    augClass = ReplaceAug(augClass);

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;
        
        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);

		if (currentAug.Class == augClass)
		{
			if (currentAug.bHasIt && currentAug.bIsActive)
				return currentAug.CurrentLevel;
			else
				return -1;
		}

		anAug = anAug.next;
	}

	return -1;
}

simulated function Float GetEnergyUsage(class<Augmentation> augClass)
{
    local float energyUse, energyMult;
    local Augmentation anAug, currentAug;
    local Augmentation PowerAug;
    
    augClass = ReplaceAug(augClass);

    energyUse = 0;
    energyMult = 1.0;

    anAug = FirstAug;
    while(anAug != None)
    {
        currentAug = anAug;
        
        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);
        
        if(currentAug.IsA('AugPower'))
            PowerAug = currentAug;

        if(currentAug.Class == augClass && currentAug.bHasIt && currentAug.bIsActive)
        {
            energyUse = currentAug.GetEnergyRate();
            if(currentAug.IsA('AugPower'))
                energyMult = currentAug.LevelValues[currentAug.CurrentLevel];
        }
        anAug = anAug.next;
    }

    if((Level.NetMode != NM_Standalone) && (PowerAug != None) && (PowerAug.bHasIt))
    {
        if((energyUse > 0) && (!PowerAug.bIsActive))
            ActivateAugByKey(PowerAug.HotKeyNum - 3);

        if((energyUse == 0) && (PowerAug.bIsActive))
            ActivateAugByKey(PowerAug.HotKeyNum - 3);

        if(PowerAug.bIsActive)				
            energyMult = PowerAug.LevelValues[PowerAug.CurrentLevel];
    }

    energyUse *= energyMult;

    return energyUse;
}

function bool ActivateAugByKey(int keyNum)
{
	local Augmentation anAug, currentAug;
	local bool bActivated;
    
    if(Player != None && Player.PlayerReplicationInfo != None)
        if(Player.PlayerReplicationInfo.bIsSpectator)
            return false;

	bActivated = False;

	if ((keyNum < 0) || (keyNum > 9))
		return False;

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;

        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);

		if ((currentAug.HotKeyNum - 3 == keyNum) && (currentAug.bHasIt))
        {
            anAug = currentAug;
			break;
        }

		anAug = anAug.next;
	}

	if (anAug == None)
	{
		player.ClientMessage(NoAugInSlot);
	}
	else
	{
		// Toggle
		if (anAug.bIsActive)
			anAug.Deactivate();
		else
			anAug.Activate();

		bActivated = True;
	}

	return bActivated;
}

function ActivateAll()
{
	local Augmentation anAug, currentAug;

    if(Player != None && Player.PlayerReplicationInfo != None)
        if(Player.PlayerReplicationInfo.bIsSpectator)
            return;

	// Only allow this if the player still has 
	// Bioleectric Energy(tm)

	if ((player != None) && (player.Energy > 0))
	{
		anAug = FirstAug;
		while(anAug != None)
		{
            currentAug = anAug;
            
            if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
                currentAug = Augmentation(currentAug.Owner);

            if((Level.NetMode == NM_Standalone) || (!currentAug.IsA('AugLight')))
                currentAug.Activate();

			anAug = anAug.next;
		}
	}
}

simulated function float GetAugLevelValue(class<Augmentation> AugClass)
{
	local Augmentation anAug, currentAug;
	local float retval;

	retval = 0;
    
    AugClass = ReplaceAug(AugClass);

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;

        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);
            
		if (currentAug.Class == augClass)
		{
			if (currentAug.bHasIt && currentAug.bIsActive)
				return currentAug.LevelValues[currentAug.CurrentLevel];
			else
				return -1.0;
		}

		anAug = anAug.next;
	}

	return -1.0;
}

function DeactivateAll()
{
	local Augmentation anAug, currentAug;

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;
        
        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);

		if (currentAug.bIsActive)
			currentAug.Deactivate();

		anAug = anAug.next;
	}
}

function CreateAugmentations(DeusExPlayer newPlayer)
{
	local int augIndex;
	local Augmentation anAug, lastAug, alternativeAug;
    local class<Augmentation> augClass, alternativeClass;
    
    ResetAugmentations(); // Copied from DXMTL

	FirstAug = None;
	LastAug  = None;

	player = newPlayer;

	for(augIndex=0; augIndex<arrayCount(augClasses); augIndex++)
	{
		if(augClasses[augIndex] != None)
		{
            augClass = augClasses[augIndex];

			anAug = Spawn(augClass, Self);
			anAug.Player = player;
            anAug.NetPriority = 1.40; // Copied from DXMTL

            alternativeClass = ReplaceAug(augClass);
            if(alternativeClass != augClass)
            {
                alternativeAug = Spawn(alternativeClass, Self);
                alternativeAug.Player = player;
                alternativeAug.NetPriority = 1.40; // Copied from DXMTL

                anAug.SetOwner(alternativeAug);
                anAug.AugmentationName = alternativeAug.AugmentationName;
                anAug.default.AugmentationName = alternativeAug.AugmentationName;
                anAug.Description = alternativeAug.Description;
                anAug.MPInfo = alternativeAug.MPInfo;
            }

			// Manage our linked list
			if (anAug != None)
			{
				if (FirstAug == None)
				{
					FirstAug = anAug;
				}
				else
				{
					LastAug.next = anAug;
				}

				LastAug  = anAug;
			}
		}
	}
}

function ResetAugmentations()
{
	local Augmentation anAug;
	local Augmentation nextAug;
    local int LocIndex;

	anAug = FirstAug;
	while(anAug != None)
	{
		nextAug = anAug.next;

        if(anAug.Owner != None && anAug.Owner.isA('Augmentation'))
            anAug.Owner.Destroy(); // Destroy the alternative too!

		anAug.Destroy();
		anAug = nextAug;
	}

	FirstAug = None;

    //Must also clear auglocs.
    for (LocIndex = 0; LocIndex < 7; LocIndex++)
    {
        AugLocs[LocIndex].AugCount = 0;
    }
}

simulated function RefreshAugDisplay()
{
	local Augmentation anAug, currentAug;
    local int i;

	if (player == None)
		return;

	// First make sure there are no augs visible in the display
	player.ClearAugmentationDisplay();

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;

        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);
            
        if(anAug != currentAug && anAug.AugmentationName != currentAug.AugmentationName)
        {
            anAug.AugmentationName = currentAug.AugmentationName;
            anAug.default.AugmentationName = currentAug.default.AugmentationName;
            anAug.Description = currentAug.Description;
            anAug.MPInfo = currentAug.MPInfo;
        }

		// First make sure the aug is active if need be
		if(currentAug.bHasIt)
		{
			if(currentAug.bIsActive)
			{
				currentAug.GotoState('Active');

				// Now, if this is an aug that isn't *always* active, then 
				// make sure it's in the augmentation display

				if (!currentAug.bAlwaysActive)
					player.AddAugmentationDisplay(currentAug);
			}
			else if ((player.bHUDShowAllAugs) && (!currentAug.bAlwaysActive))
			{
				player.AddAugmentationDisplay(currentAug);		
			}
		}

		anAug = anAug.next;
	}
}

simulated function int NumAugsActive()
{
	local Augmentation anAug, currentAug;
	local int count;

	if (player == None)
		return 0;

	count = 0;
	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;

        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);

		if (currentAug.bHasIt && currentAug.bIsActive && !currentAug.bAlwaysActive)
			count++;

		anAug = anAug.next;
	}

	return count;
}

function SetPlayer(DeusExPlayer newPlayer)
{
	local Augmentation anAug, currentAug;

	player = newPlayer;

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;

        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);

		currentAug.player = player;
		anAug = anAug.next;
	}
}

function BoostAugs(bool bBoostEnabled, Augmentation augBoosting)
{
	local Augmentation anAug, currentAug;

	anAug = FirstAug;
	while(anAug != None)
	{
        currentAug = anAug;

        if(currentAug.Owner != None && currentAug.Owner.isA('Augmentation'))
            currentAug = Augmentation(currentAug.Owner);

		// Don't boost the augmentation causing the boosting!
		if(currentAug != augBoosting)
		{
			if(bBoostEnabled)
			{
				if(currentAug.bIsActive && !currentAug.bBoosted && (currentAug.CurrentLevel < currentAug.MaxLevel))
				{
					currentAug.Deactivate();
					currentAug.CurrentLevel++;
					currentAug.bBoosted = True;
					currentAug.Activate();
				}
			}
			else if (currentAug.bBoosted)
			{
				currentAug.CurrentLevel--;
				currentAug.bBoosted = False;
			}
		}
		anAug = anAug.next;
	}
}

function class<Augmentation> ReplaceAug(class<Augmentation> augClass)
{
    /*if(augClass == Class'DeusEx.AugRadarTrans')
        return Class'ANNAAugNanobots';
    else if(augClass == Class'DeusEx.AugDefense')
        return Class'ANNAAugDefense';*/
    return augClass;
}

function bool specialAug(Augmentation anAug)
{
    if(anAug.IsA('AugBallistic'))
        return true;
    else if(anAug.IsA('AugSpeed'))
        return true;
    else if(anAug.IsA('ANNAAugDefense'))
        return true;
    else if(anAug.IsA('AugAqualung'))
        return true;
    else if(anAug.IsA('AugEMP'))
        return true;
    else if(anAug.IsA('AugShield'))
        return true;
    else if(anAug.IsA('AugEnviro'))
        return true;
    else
        return false;
}

defaultproperties
{
}
