//=============================================================================
// _ANNAPlayer. (C) 2012 "Dei" & Nobody
//=============================================================================
class _ANNAPlayer extends MTLPlayer abstract;

var float defaultMaxFrobDistance;
var bool bSpectatingEnabled;
var ANNAManager Manager;

const BOOL_FALSE = 0;
const BOOL_TRUE = 1;

var float spectateStartTime;
var float reconnectStartTime;
var float disconnectStartTime;
var float suicideStartTime;

var bool bQueuedSpectate;
var bool bQueuedReconnect;
var bool bQueuedDisconnect;
var bool bQueuedSuicide;

var float _timerSecondsPrev;
var int FrameCounter;
var int FPS;
var float _timerSeconds, clientTimeSeconds;
var string errorMessage;

var string messageHistory[32];
var float messageHistoryTimes[32];
var int messageHistoryCount[32];
var string _customMenuMain;

var int _HealthHeadReplicated;
var int _HealthTorsoReplicated;
var int _HealthLegsReplicated;

var float lastTeamHeal;

replication
{
    reliable if(ROLE == ROLE_Authority && bNetOwner)
        clientStopFiring, clientUpdateURL, clientReconnect, clientDisconnect, defaultMaxFrobDistance, bSpectatingEnabled, _customMenuMain, _WeaponSimIdle;

    reliable if(ROLE == ROLE_Authority)
        _HealthHeadReplicated, _HealthTorsoReplicated, _HealthLegsReplicated;

    reliable if (Role < ROLE_Authority)
        Spectate, serverPlayerPossess, serverSetName, serverShowInventoryWindow, serverShowSkillsWindow, serverShowAugmentationsWindow, _serverFPS, _unrecognizedCommand;
}

simulated event PostRender(canvas Canvas)
{
    local float _timeDifference, _timerSecondsCorrected;

    if(Len(errorMessage) == 0)
        errorMessage = Localize("Errors","Exec","Core");

    //if(PlayerReplicationInfo != None)
    //    _timerSecondsCorrected = _timerSeconds+(PlayerReplicationInfo.Ping/1000);
    //else
    //    _timerSecondsCorrected = _timerSeconds;

    FrameCounter++;
    _timeDifference = _timerSeconds-_timerSecondsPrev;
    if(_timeDifference >= 1)
    {
        FPS = int(FrameCounter/_timeDifference);
        FrameCounter = 0;

        _serverFPS(FPS, _timeDifference);

        _timerSecondsPrev = _timerSeconds;
    }
/*
    if(Len(Player.Console.MsgText[Player.Console.TopLine]) > 0 && Player.Console.MsgText[Player.Console.TopLine] == errorMessage)
    {
        unrecognizedCommand(Player.Console.History[Player.Console.HistoryCur-1]);
        Player.Console.MsgText[Player.Console.TopLine] = "";
    }
*/
	Super.PostRender(Canvas);
}

function _unrecognizedCommand(string Command)
{
    local int _bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.PlayerCommand(self, Command, _bOverride);
    if(_bOverride != BOOL_TRUE)
        ClientMessage(Localize("Errors","Exec","Core"));
}

function _serverFPS(int _newFPS, float _timerDiffClient)
{
    local float _timerDiffServer, _timerSecondsDiff;
    local int _DT;
    
    // Log("_timerDiffClient"@_timerDiffClient);

    _timerDiffServer = _timerSeconds-_timerSecondsPrev;
    // Log("_timerSeconds"@_timerSeconds);
    // Log("_timerSecondsPrev"@_timerSecondsPrev);
    // Log("_timerDiffServer"@_timerDiffServer);
    _timerSecondsDiff = _timerDiffClient/_timerDiffServer; // Should be ~1 second
    // Log("_timerSecondsDiff"@_timerSecondsDiff);
    _timerSecondsPrev = _timerSeconds;

    if(_timerSecondsDiff > 0)
        _DT = Int(_timerSecondsDiff*100);
    else
        _DT = -1;

    // Log("_DT"@_DT);
    FPS = Int(_newFPS*_timerSecondsDiff);
    if(EMMAPRI(PlayerReplicationInfo) != None)
    {
        EMMAPRI(PlayerReplicationInfo).FPS = FPS;
        EMMAPRI(PlayerReplicationInfo).DT = _DT;
    }
}

/**
  Spawning the AntiCheat actor.
*/
function PostBeginPlay()
{
    Super.PostBeginPlay();

    Spawn(class'_ANNAAntiCheat', self);

    if(Manager == None)
        Manager = findManager();

    if(Manager != None)
    {
        defaultMaxFrobDistance = Manager.MaxFrobDistance;
        _customMenuMain = Manager.customMenuMain;
    }

    //clientTimer();
    
}

/**
    Fix for grenade-plant exploit. Different approach.
*/
function UpdateBeltText(Inventory item)
{
    Super.UpdateBeltText(item);
    if(DeusExWeapon(item) != None)
        if(DeusExWeapon(item).ProjectileClass != None && ClassIsChildOf(DeusExWeapon(item).ProjectileClass, Class'ThrownProjectile'))
            if(DeusExWeapon(item).bDestroyOnFinish && (DeusExWeapon(item).AmmoType == None || DeusExWeapon(item).AmmoType.AmmoAmount <= 0))
                DeusExWeapon(item).Destroy();
}

/**
  Find and return the ANNAManager
*/
function ANNAManager findManager()
{
    local ANNAManager _curManager;

    if(Manager != None)
        return Manager;

    foreach AllActors(class'ANNAManager', _curManager)
        if(_curManager != None)
            return _curManager;

    return none;
}

/**
  Before we actually frob our target, reset the MaxFrobDistance.
*/
function DoFrob(Actor Frobber, Inventory frobWith)
{
    HighlightCenterObject();
    Super.DoFrob(Frobber, frobWith);
}

/**
  Reset the MaxFrobDistance
*/
function HighlightCenterObject()
{
    MaxFrobDistance = defaultMaxFrobDistance;
    Super.HighlightCenterObject();
}

/**
  Make sure the player uses our customized AugmentationManager 
*/
function InitializeSubSystems()
{
    local Skill _VAC;

    if(Manager == None)
        Manager = findManager();

    bSpectatingEnabled = Manager.bEnableSpectating;

    if((Level.NetMode == NM_Standalone) && (BarkManager == None))
        BarkManager = Spawn(Class'BarkManager', self);

    CreateColorThemeManager();
    if(ThemeManager != None)
        ThemeManager.SetOwner(self);

    if(Level.NetMode != NM_Standalone)
    {
        if((AugmentationSystem != None) && !AugmentationSystem.IsA('_ANNAAugmentationManager'))
        {
            AugmentationSystem.ResetAugmentations();
            AugmentationSystem.Destroy();
            AugmentationSystem = None;
        }

        if((SkillSystem != None) && !SkillSystem.IsA('CBPSkillManager'))
        {
            _VAC = SkillSystem.FirstSkill;

            while(_VAC != None)
            {
                _VAC.Destroy();
                _VAC = _VAC.Next;
            }
            SkillSystem.Destroy();
            SkillSystem = None;
        }
    }

    if(AugmentationSystem == None)
    {
        AugmentationSystem = Spawn(Class'_ANNAAugmentationManager', self);
        AugmentationSystem.CreateAugmentations(self);
        AugmentationSystem.AddDefaultAugmentations();
    }
    else
    {
        AugmentationSystem.SetPlayer(self);
        AugmentationSystem.SetOwner(self);
    }

    if(SkillSystem == None)
    {
        SkillSystem = Spawn(Class'CBPSkillManager',self);
        SkillSystem.CreateSkills(self);
    }
    else
    {
        SkillSystem.SetPlayer(self);
        SkillSystem.SetOwner(self);
    }

    if((Level.NetMode == NM_Standalone) || !bBeltIsMPInventory)
        CreateKeyRing();
}

/**
  Make sure the player uses our customized HUD 
*/
event Possess()
{
    local DeusExRootWindow _root;

    Super.Possess();

    _root = DeusExRootWindow(RootWindow);
    if(_root != None)
    {
        if(_root.HUD != None)
            _root.HUD.Destroy();

        _root.HUD = _ANNAHUD(_root.NewChild(Class'_ANNAHUD'));
        _root.HUD.UpdateSettings(self);
        _root.HUD.SetWindowAlignments(HALIGN_Full, VALIGN_Full, 0.00, 0.00);
    }

    Spawn(class'_ANNATimer', self);
	if(PlayerIsRemoteClient())
        serverPlayerPossess();
}

function serverPlayerPossess()
{
    if(Manager == None)
        Manager = findManager();

    Manager.PlayerPossess(self);
}

/**
  Make sure the player uses our new scoreboard for the multiplayer-win screen.
*/
function ShowMultiplayerWin(String winnerName, int winningTeam, String Killer, String Killee, String Method)
{
    local _ANNAHUDMP _mpScr;
    local DeusExRootWindow _root;

    if((Player != None) && (Player.Console != None))
        Player.Console.ClearMessages();

    _root = DeusExRootWindow(rootWindow);

    if(_root != None)
    {
        _mpScr = _ANNAHUDMP(_root.InvokeUIScreen(Class'_ANNAHUDMP', True));
        _root.MaskBackground(True);

        if(_mpScr != None)
        {
            _mpScr.winnerName = winnerName;
            _mpScr.winningTeam = winningTeam;
            _mpScr.winKiller = Killer;
            _mpScr.winKillee = Killee;
            _mpScr.winMethod = Method;
        }
    }

    if(PlayerIsClient())
        if(AugmentationSystem != None)
            AugmentationSystem.DeactivateAll();
}

/**
  New <i>server -> client</i> function to tell the client to stop with firing.
  In this case the client is supposed to stop, if the player jumps into the water.
*/
simulated function clientStopFiring()
{
    DeusExWeapon(inHand).GotoState('SimFinishFire');
    DeusExWeapon(inHand).PlayIdleAnim();
}

simulated function clientUpdateURL(string NewOption, string NewValue, bool bSaveDefault, optional bool bSaveConfig)
{
    UpdateURL(NewOption, NewValue, bSaveDefault);
    if(bSaveConfig)
        SaveConfig();
}

function clientChangeTeam(int N)
{
    clientUpdateURL("Team", string(N), true, true);
}

/**
  Extended state swimming for fixing the water bug.
*/
state PlayerSwimming
{
    /**
    Checks if the weapon is able to work under water, if not it forces the client to stop with firing.
    */
    function BeginState()
    {
        local Class<ThrownProjectile> _thrownProjectileClass;
        local bool _bOverride;
        _bOverride = false;

        Super.BeginState();
        if(DeusExWeapon(inHand) != None)
        {
            if(DeusExWeapon(inHand).ProjectileClass != None)
            {
                _thrownProjectileClass = Class<ThrownProjectile>(DeusExWeapon(inHand).ProjectileClass);
                if(_thrownProjectileClass != None)
                    _bOverride = true;
            }
            if(((DeusExWeapon(inHand).EnviroEffective == ENVEFF_Air) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_Vacuum) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_AirVacuum)) && !_bOverride)
            {
                DeusExWeapon(inHand).GotoState('FinishFire');
                clientStopFiring();
            }
        }
    }

    /**
    Modified fire command-function which just makes the weapon firing, if it's able to fire under water.
    So that the player can't try to continue firing under water if he presses the fire button again.
    */
    exec function Fire(optional float _F)
    {
        local Class<ThrownProjectile> _thrownProjectileClass;
        local bool _bOverride;
        _bOverride = false;

        if(DeusExWeapon(inHand) != None)
        {
            if(DeusExWeapon(inHand).ProjectileClass != None)
            {
                _thrownProjectileClass = Class<ThrownProjectile>(DeusExWeapon(inHand).ProjectileClass);
                if(_thrownProjectileClass != None)
                    _bOverride = true;
            }

            if(((DeusExWeapon(inHand).EnviroEffective == ENVEFF_Air) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_Vacuum) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_AirVacuum)) && !_bOverride)
                return;
        }
        Super.Fire(_F);
    }
}

/**
  Only make PopHealth available in singleplayer games. In multiplayer it could be abused for 'god mode'.
*/
function PopHealth(float _health0, float _health1, float _health2, float _health3, float _health4, float _health5)
{
    if(Level.NetMode == NM_StandAlone)
        Super.PopHealth(_health0, _health1, _health2, _health3, _health4, _health5);
}

/**
  Only make ServerReStartGame available in singleplayer games. In multiplayer it could crash the server.
*/
function ServerReStartGame()
{
    if(Level.NetMode == NM_StandAlone)
        Level.Game.RestartGame();
}

/**
  MaintainEnergy could generate energy when a negative deltaTime was given.
*/
function MaintainEnergy(float _deltaTime)
{
    if(_deltaTime > 0)
        Super.MaintainEnergy(_deltaTime);
}

/**
  Most difficult and complex anti cheat. Disable the editactor command.
*/
exec function EditActor();

/**
  Disable ShowPath command, can be used to create log-spam & serverlag.
*/
exec function ShowPath();

/**
  Check if the spydrone augmentation is actually active.
*/
function CreateDrone()
{
    if(bSpyDroneActive)
    {
        Super.CreateDrone();
        
        if(aDrone != None)
            aDrone.bBlockPlayers = true;
    }
}

/**
  Make sure the item we drop is actually ours.
*/
exec function bool DropItem(optional Inventory _inv, optional bool _bDrop)
{
	local DeusExWeapon _Item;
    local Inventory _InventoryItem;
    local int _bOverride;
    
    _InventoryItem = _inv;
	if(_inv != None)
		_Item = DeusExWeapon(_inv);
	else if(inHand != None)
    {
        _Item = DeusExWeapon(inHand);
        _InventoryItem = _Item;
    }

    if(_inv != None && _inv.Owner != self)
        return false;
        
    if(Manager == None)
        Manager = findManager();

    Manager.DropItem(self, _InventoryItem, _bOverride);

    if(_bOverride == BOOL_TRUE)
        return false;

    if(_Item != None)
        if(_Item.bHandToHand && (_Item.IsInState('Idle') || _Item.IsInState('Idle2') || _Item.IsInState('DownWeapon') || _Item.IsInState('Reload'))) // Fix invisible/bugged hand to hand weapons.
            _WeaponSimIdle(_Item);

    return Super.DropItem(_inv, _bDrop);
}

simulated function _WeaponSimIdle(DeusExWeapon _Item)
{
    if(_Item != None)
        _Item.GoToState('SimIdle');
}

/**
  Make sure the player uses our main menu
*/
exec function ShowMainMenu()
{
    local DeusExRootWindow _root;
    local DeusExLevelInfo _info;
    local MissionEndgame _Script;
    local class<DeusExBaseWindow> _newMenuMainClass;

    if(bIgnoreNextShowMenu)
    {
        bIgnoreNextShowMenu = False;
        return;
    }

    if(Level.NetMode == NM_Standalone)
        _info = GetLevelInfo();

    ConsoleCommand("FLUSH");

    if((_info != None) && (_info.MissionNumber == 98)) 
    {
        bIgnoreNextShowMenu = True;
        PostIntro();
    }
    else if((_info != None) && (_info.MissionNumber == 99))
    {
        foreach AllActors(class'MissionEndgame', _Script)
            break;

        if(_Script != None)
            _Script.FinishCinematic();
    }
    else
    {
        _root = DeusExRootWindow(rootWindow);

        if(_root != None)
        {
            if(Len(_customMenuMain) > 0)
            {
                _newMenuMainClass = class<DeusExBaseWindow>(DynamicLoadObject(_customMenuMain, class'Class'));

                if(_newMenuMainClass != None)
                {
                    _root.InvokeMenu(_newMenuMainClass);
                    return;
                }
            }
            _root.InvokeMenu(Class'_ANNAMenuMain');
        }
    }
}

function GiveInitialInventory()
{
    local int _bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.GiveInitialInventory(self, _bOverride);
    if(_bOverride != BOOL_TRUE)
        Super.GiveInitialInventory();
}

/**
  Check and (re-)set skin. Only applies to team based games.
*/
simulated function CheckSkin()
{
    Mesh = Default.Mesh;
    DrawScale = Default.DrawScale;
    MultiSkins[0] = Default.MultiSkins[0];
    MultiSkins[1] = Default.MultiSkins[1];
    MultiSkins[2] = Default.MultiSkins[2];
    MultiSkins[3] = Default.MultiSkins[3];
    MultiSkins[4] = Default.MultiSkins[4];
    MultiSkins[5] = Default.MultiSkins[5];
    MultiSkins[6] = Default.MultiSkins[6];
    MultiSkins[7] = Default.MultiSkins[7];

    V7C = false;
}

/**
  Check & fix inventory: projectiles don't have an owner when the player crashes and returns.
*/
function CheckInventory()
{
    local Inventory _Inv;

    if(Inventory != None)
        for(_Inv = Inventory; _Inv != None; _Inv = _Inv.Inventory)
            _Inv.Instigator = self;
}

/**
  Added support for teamsay messages in the MessagingSpectator.
*/
exec function TeamSay(string Z38)
{
    local Pawn P, Player;
    local string str;
    local int i;
    local bool bMessageFound, bSpamDetected;

    if(V7D)
        return;

    if(TeamDMGame(DXGame) == None)
    {
        Say(Z38);
        return;
    }

    if(!PlayerIsListenClient() && (Level.NetMode != NM_Standalone))
    {
        V62(Len(Z38));
        if(V4A())
            return;
    }

    if(Z38 != "")
        V54(Z38,True);

    if(Z38 == "")
        return;
        
    for(i = 0; i < 32; i++)
    {
        if(Len(messageHistory[i]) > 0)
        {
            if(Level.TimeSeconds-messageHistoryTimes[i] >= 20)
            {
                messageHistory[i] = "";
                messageHistoryTimes[i] = 0;
                messageHistoryCount[i] = 0;
            }
            else if(messageHistory[i] == Z38)
            {
                bMessageFound = true;
                messageHistoryTimes[i] = Level.TimeSeconds;

                // So we already send this message at least once in the previous 20 seconds.
                messageHistoryCount[i]++;

                if(messageHistoryCount[i] == 3)
                    ClientMessage("|cFF8000Warning: it seems as if you're spamming!");
                else if(messageHistoryCount[i] > 3)
                {
                    ClientMessage("|cFF0000Warning: it seems as if you're spamming!");
                    bSpamDetected = true;
                }
                break;
            }
        }
    }

    if(!bMessageFound)
    {
        for(i = 0; i < 32; i++)
        {
            if(Len(messageHistory[i]) == 0)
            {
                messageHistory[i] = Z38;
                messageHistoryTimes[i] = Level.TimeSeconds;
                messageHistoryCount[i] = 1;
                break;
            }
        }
    }

    str = V50() $ ":" @ Z38;

    if(bSpamDetected)
    {
        ClientMessage(str, 'TeamSay', True);
    }
    else
    {
        if(Role == 4)
            Log(str, 'TeamSay');

        for(P=Level.PawnList; P!=None; P=P.NextPawn)
        {
            if((P.IsA('MTLPlayer') && (P.PlayerReplicationInfo.Team == PlayerReplicationInfo.Team)))
            {
                P.ClientMessage(str, 'TeamSay', True);
            }
            else if(P.IsA('MessagingSpectator'))
            {
                if(PlayerReplicationInfo.Team == 0) // TEAM_UNATCO
                    P.ClientMessage(str,'TeamUNATCOSay', True);
                else if(PlayerReplicationInfo.Team == 1) // TEAM_NSF
                    P.ClientMessage(str,'TeamNSFSay', True);
            }
        }
    }
}

exec function Say(string Z38)
{
	local Pawn P;
	local string str;
    local int i;
    local bool bMessageFound, bSpamDetected;

	if(V7D)
		return;

	if (!PlayerIsListenClient() && (Level.NetMode != NM_Standalone))
	{
		V62(Len(Z38));
		if ( V4A() )
		{
			return;
		}
	}
	if(Z38 != "")
		V54(Z38,True);

	if(Z38 == "")
		return;
        
    for(i = 0; i < 32; i++)
    {
        if(Len(messageHistory[i]) > 0)
        {
            if(Level.TimeSeconds-messageHistoryTimes[i] >= 20)
            {
                messageHistory[i] = "";
                messageHistoryTimes[i] = 0;
                messageHistoryCount[i] = 0;
            }
            else if(messageHistory[i] == Z38)
            {
                bMessageFound = true;
                messageHistoryTimes[i] = Level.TimeSeconds;

                // So we already send this message at least once in the previous 20 seconds.
                messageHistoryCount[i]++;

                if(messageHistoryCount[i] == 3)
                    ClientMessage("|cFF8000Warning: it seems as if you're spamming!");
                else if(messageHistoryCount[i] > 3)
                {
                    ClientMessage("|cFF0000Warning: it seems as if you're spamming!");
                    bSpamDetected = true;
                }
                break;
            }
        }
    }

    if(!bMessageFound)
    {
        for(i = 0; i < 32; i++)
        {
            if(Len(messageHistory[i]) == 0)
            {
                messageHistory[i] = Z38;
                messageHistoryTimes[i] = Level.TimeSeconds;
                messageHistoryCount[i] = 1;
                break;
            }
        }
    }

	str = V50() $ ":" @ Z38;

    if(bSpamDetected)
    {
        ClientMessage(str, 'Say', True);
    }
    else
    {
        if(Role == 4)
            Log(str, 'Say');

        for(P=Level.PawnList; P!=None; P=P.NextPawn)
        {
            if ( P.IsA('MTLPlayer') || P.IsA('MessagingSpectator') )
            {
                P.ClientMessage(str, 'Say', True);
            }
        }
    }
}

exec function Spectate()
{
    if(Manager == None)
        Manager = findManager();

    if(Manager.bEnableSpectating)
    {
        if(spectateStartTime < Level.TimeSeconds)
        {
            spectateStartTime = Level.TimeSeconds+2; // We start in 2 seconds.
            bQueuedSpectate = true;
            ClientMessage("You will start spectating in a few seconds.");
        }
    }
}
/*
exec function Reconnect()
{
    if(reconnectStartTime < Level.TimeSeconds)
    {
        reconnectStartTime = Level.TimeSeconds+2; // We reconnect in 2 seconds.
        bQueuedReconnect = true;
        ClientMessage("You will reconnect in a few seconds.");
    }
}

exec function Disconnect()
{
    if(disconnectStartTime < Level.TimeSeconds)
    {
        disconnectStartTime = Level.TimeSeconds+1; // We disconnect in 1 second.
        bQueuedDisconnect = true;
        ClientMessage("You will disconnect in a few seconds.");
    }
}*/

exec function Suicide()
{
    if((DeusExMPGame(Level.Game) != None) && DeusExMPGame(Level.Game).bNewMap)
        return;

    if(bNintendoImmunity || (NintendoImmunityTimeLeft > 0.00))
        return;

    if(suicideStartTime < Level.TimeSeconds)
    {
        suicideStartTime = Level.TimeSeconds+1; // We suicide in 1 second.
        bQueuedSuicide = true;
    }
}

function MultiplayerTick(float DeltaTime)
{
    Super.MultiplayerTick(DeltaTime);

    if(bQueuedSpectate && (Level.Timeseconds >= spectateStartTime))
    {
        bQueuedSpectate = false;
        GotoState('PlayerSpectating');
        BroadcastMessage(PlayerReplicationInfo.PlayerName@"started spectating.");
        ClientMessage("You are now spectating. Use the left & right mouse button to switch between player views.");
    }

    if(bQueuedReconnect && (Level.Timeseconds >= (reconnectStartTime)))
    {
        bQueuedReconnect = false;
        clientReconnect();
    }

    if(bQueuedDisconnect && (Level.Timeseconds >= (disconnectStartTime)))
    {
        bQueuedDisconnect = false;
        clientDisconnect();
    }

    if(bQueuedSuicide && (Level.Timeseconds >= (suicideStartTime)))
    {
        bQueuedSuicide = false;
        CreateKillerProfile(None, 0, 'None', "");
        KilledBy(None);
    }
}

function KilledBy(pawn EventInstigator)
{
    bQueuedSuicide = false;
    Super.KilledBy(EventInstigator);
}

simulated function clientReconnect()
{
    local _ANNADummy _dummy;
    
    _dummy = Spawn(Class'_ANNADummy');
    if(_dummy != None)
        _dummy.ConsoleCommand("reconnect");
}

simulated function clientDisconnect()
{
    local _ANNADummy _dummy;
    
    _dummy = Spawn(Class'_ANNADummy');
    if(_dummy != None)
        _dummy.ConsoleCommand("disconnect");
}

state PlayerSpectating
{
    ignores SeePlayer, HearNoise, Bump, TakeDamage, Died, ZoneChange, FootZoneChange, ActivateAugmentation, BuySkills;

    exec function Suicide();
    exec function AltFire(optional float F);
    exec function God();
    exec function BehindView(bool B);
    exec function Invisible(bool B);
    exec function Ghost();
    exec function Walk();
    exec function Fly();

    exec function TeamSay(string Z38)
    {
        Say(Z38);
    }

    exec function Fire(optional float F)
    {
    }

    function ChangeTeam( int N )
    {
        Level.Game.ChangeTeam(self, N);
    }

    exec function Spectate()
    {
        ReducedDamageType = '';
        if(Level.Game.bTeamGame)
        {
            if(TeamDMGame(Level.Game) != None)
                PlayerReplicationInfo.Team = TeamDMGame(Level.Game).GetAutoTeam();

            if(PlayerReplicationInfo.Team == 0) // TEAM_UNATCO
                BroadcastMessage(PlayerReplicationInfo.PlayerName@"joined UNATCO.");
            else if(PlayerReplicationInfo.Team == 1) // TEAM_NSF
                BroadcastMessage(PlayerReplicationInfo.PlayerName@"joined NSF.");
            else // TEAM_UHH?
                BroadcastMessage(PlayerReplicationInfo.PlayerName@"started playing.");
        }
        else
        {
            PlayerReplicationInfo.Team = 128;
            BroadcastMessage(PlayerReplicationInfo.PlayerName@"started playing.");
        }
        clientUpdateURL("Team", string(PlayerReplicationInfo.Team), true);
        V7C = false;

        bVisionImportant = true;
        PlayerReplicationInfo.Score = 0.0;
        PlayerReplicationInfo.Deaths = 0.0;
        PlayerReplicationInfo.Streak = 0.0;
        PlayerReplicationInfo.bIsSpectator = false;
        SkillPointsAvail = 0;
        SkillPointsTotal = 0;

        Level.Game.RestartPlayer(self);

        ResetBasedPawnSize();
        bDetectable = true;
        bHidden = false;
        SetCollision(true,true,true);

        GotoState('PlayerWalking');
    }

    function SendVoiceMessage(PlayerReplicationInfo Sender, PlayerReplicationInfo Recipient, name messagetype, byte messageID, name broadcasttype)
    {
    }

    exec function ParseLeftClick()
    {
        local Pawn P;
        local bool bEnd, bNotifyPlayer;

        bEnd = false;

        if(_ANNAPlayer(ViewTarget) != None)
            P = _ANNAPlayer(ViewTarget);
        else
            P = Level.PawnList;

        while(P != None)
        {
            if(_ANNAPlayer(P) != None && !P.PlayerReplicationInfo.bIsSpectator)
            {
                if(P != ViewTarget && P != self)
                {
                    if(ViewTarget == None)
                        bNotifyPlayer = true;

                    ViewTarget = P;
                    ViewTarget.BecomeViewTarget();
                    ClientMessage("You are now viewing from"@_ANNAPlayer(ViewTarget).PlayerReplicationInfo.PlayerName$"("$_ANNAPlayer(ViewTarget).PlayerReplicationInfo.PlayerID$")");

                    if(bNotifyPlayer)
                        ClientMessage("To go back to third person spectating, use any of your movement keys.");

                    break;
                }
                else if(P == ViewTarget && bEnd)
                    break;
            }
            P = P.nextPawn;
            if(P == None && !bEnd)
            {
                bEnd = true; // We reached the end of the list, so we start from the beginning...
                P = Level.PawnList;
            }
        }
    }

    exec function ParseRightClick()
    {
        local _ANNAPlayer prevPlayer;
        local Pawn P;
        local bool bNotifyPlayer;

        P = Level.PawnList;

        while(P != None)
        {
            if(_ANNAPlayer(P) != None && !P.PlayerReplicationInfo.bIsSpectator)
            {
                if(P != self)
                {
                    if(P == ViewTarget && prevPlayer != None)
                    {
                        if(ViewTarget == None)
                            bNotifyPlayer = true;

                        ViewTarget = prevPlayer;
                        ViewTarget.BecomeViewTarget();
                        ClientMessage("You are now viewing from"@_ANNAPlayer(ViewTarget).PlayerReplicationInfo.PlayerName$"("$_ANNAPlayer(ViewTarget).PlayerReplicationInfo.PlayerID$")");

                        if(bNotifyPlayer)
                            ClientMessage("To go back to third person spectating, use any of your movement keys.");

                        break;
                    }
                    else if(P != ViewTarget)
                    {
                        prevPlayer = _ANNAPlayer(P);
                    }
                }
            }
            P = P.nextPawn;
        }

        if(prevPlayer != None)
        {
            ViewTarget = prevPlayer;
            ViewTarget.BecomeViewTarget();
            ClientMessage("You are now viewing from"@_ANNAPlayer(ViewTarget).PlayerReplicationInfo.PlayerName$"("$_ANNAPlayer(ViewTarget).PlayerReplicationInfo.PlayerID$")");
        }
    }

    function bool RestrictInput()
    {
        return true;
    }

    function ProcessMove(float DeltaTime, vector NewAccel, eDodgeDir DodgeMove, rotator DeltaRot)
    {
        local _ANNAPlayer P;
        P = _ANNAPlayer(ViewTarget);

        if(P != None)
        {
            if(P.PlayerReplicationInfo.bIsSpectator || (NewAccel.X > 0 || NewAccel.Y > 0 || NewAccel.Z > 0))
            {
                SetLocation(ViewTarget.Location);
                SetRotation(ViewTarget.Rotation);
                bBehindView = false;
                ViewTarget = None;
            }
        }

        Acceleration = Normal(NewAccel);
        Velocity = Normal(NewAccel) * 300;
        AutonomousPhysics(DeltaTime);
    }
    
    simulated function MultiplayerTickClient()
    {

        if(ViewTarget != None)
        {
            SetLocation(ViewTarget.Location);
            SetRotation(ViewTarget.Rotation);
        }
    }
    
	function MultiplayerTick(float DeltaTime)
	{
		if(Role < ROLE_Authority)
		{
			MultiplayerTickClient();
			return;
		}

        Super.MultiplayerTick(DeltaTime);
    }

    event PlayerTick(float DeltaTime)
    {
        RefreshSystems(DeltaTime);
		MultiplayerTick(DeltaTime);
        UpdateTimePlayed(DeltaTime);
        if(bUpdatePosition)
            ClientUpdatePosition();

        PlayerMove(DeltaTime);
    }

    function PlayerMove(float DeltaTime)
    {
        local rotator newRotation;
        local vector X,Y,Z;

        GetAxes(ViewRotation,X,Y,Z);

        aForward *= 0.1;
        aStrafe  *= 0.1;
        aLookup  *= 0.24;
        aTurn    *= 0.24;
        aUp		 *= 0.1;

        Acceleration = aForward*X + aStrafe*Y + aUp*vect(0,0,1);  

        UpdateRotation(DeltaTime, 1);

        if(Role < ROLE_Authority) // then save this move and replicate it
            ReplicateMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
        else
            ProcessMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
    }

    function EndState()
    {
    }

    function spectateInitialize()
    {
        local SecurityCamera camera;
        local AutoTurret turret;
        local DeusExDecoration decoration;
        local ThrownProjectile projectile;

        foreach AllActors(Class'SecurityCamera', camera)
        {
            if(camera.safeTarget == self)
            {
                camera.safeTarget = None;
                camera.Team = -1;
            }
        }

        foreach AllActors(Class'AutoTurret', turret)
        {
            if(turret.safeTarget == self)
            {
                turret.safeTarget = None;
                turret.Team = -1;
            }
        }

        foreach AllActors(Class'ThrownProjectile', projectile)
        {
            if(projectile.Owner == self)
            {
                projectile.bDisabled = true;
                projectile.Team = -1;
            }
        }

        foreach AllActors(Class'DeusExDecoration', decoration)
        {
            if(decoration.Owner == self)
            {
                decoration.SetOwner(None);
            }
        }
    }

    function BeginState()
    {
        local Inventory Inv;
        local DeusExWeapon W;
        local Pawn P;

        W = DeusExWeapon(Weapon);
        if(W != None)
            W.ScopeOff();

        bBuySkills = false;

        DropDecoration();
        if(inHand != None)
            SetInHand(None);

        spectateInitialize();

        if(Inventory != None)
        {
            for(Inv = Inventory; Inv != None; Inv = Inv.Inventory)
            {
                DeleteInventory(Inv);
                Inv.Destroy();
            }
        }

        if(invulnSph != None)
        {
            invulnSph.Destroy();
            invulnSph = None;
        }

        P = Level.PawnList;

        while(P != None)
        {
            if(_ANNAPlayer(P) != None)
            {
                if(_ANNAPlayer(P) != self)
                {
                    if(_ANNAPlayer(P).ViewTarget == self) // The player is spectating us, while we're about to spectate
                    {
                        _ANNAPlayer(P).SetLocation(Location);
                        _ANNAPlayer(P).SetRotation(Rotation);
                        _ANNAPlayer(P).bBehindView = false;
                        _ANNAPlayer(P).ViewTarget = None;
                    }
                }
            }
            P = P.nextPawn;
        }

        if(AugmentationSystem != None)
        {
            AugmentationSystem.ResetAugmentations();
            // AugmentationSystem.DeactivateAll
        }

        DeusExRootWindow(RootWindow).HUD.belt.ClearBelt();

        bNintendoImmunity = False;
        NintendoImmunityTimeLeft = 0.0;
        ViewTarget = None;
        KillShadow();

        bBehindView = False;
        FrobTarget = None;
        ReducedDamageType = 'All';
        bVisionImportant = false;
        SetBasedPawnSize(Default.CollisionRadius, 30.0);
        PlayerReplicationInfo.Score = 0.0;
        PlayerReplicationInfo.Deaths = 0.0;
        PlayerReplicationInfo.Streak = 0.0;
        PlayerReplicationInfo.bIsSpectator = true;
        PlayerReplicationInfo.Team = 100;
        SkillPointsAvail = 0;
        SkillPointsTotal = 0;
        bDetectable = false;
        bHidden = true;
        SetCollision(false,false,false);
        bCollideWorld = true;
        SetPhysics(PHYS_Flying);
    }
}

function ForceDisconnect(string Message)
{
    if(Player != None && Player.Console != None)
        Player.Console.AddString(Message);

    DisconnectPlayer();
}

function bool DXReduceDamage(int Damage, name damageType, vector hitLocation, out int adjustedDamage, bool bCheckOnly)
{
	local float prevDamage, newDamage, damageEnergyDrain;
	local float augLevel, skillLevel;
	local float pct;
	local HazMatSuit suit;
	local BallisticArmor armor;
	local bool bReduced;

	bReduced = False;
	newDamage = Float(Damage);

	if((damageType == 'TearGas') || (damageType == 'PoisonGas') || (damageType == 'Radiation') || (damageType == 'HalonGas') || (damageType == 'PoisonEffect') || (damageType == 'Poison'))
	{
		if(AugmentationSystem != None)
			augLevel = AugmentationSystem.GetAugLevelValue(class'AugEnviro');

		if(augLevel >= 0.0)
        {
            prevDamage = newDamage;
			newDamage *= augLevel;
            //if(Manager.bDrainEnergyOnUsage)
            //    damageEnergyDrain += (prevDamage-newDamage)*(_ANNAAugmentationManager(AugmentationSystem).GetEnergyUsage(class'AugEnviro') / 90);
            
        }

		if(newDamage ~= 0.0)
		{
			StopPoison();
			drugEffectTimer -= 4;	// stop the drunk effect
			if (drugEffectTimer < 0)
				drugEffectTimer = 0;
		}

        if(UsingChargedPickup(class'HazMatSuit'))
		{
			skillLevel = SkillSystem.GetSkillLevelValue(class'SkillEnviro');
			newDamage *= 0.75 * skillLevel;
		}
	}
	else if((damageType == 'Shot') || (damageType == 'Sabot') || (damageType == 'Exploded') || (damageType == 'AutoShot'))
	{
        if(UsingChargedPickup(class'BallisticArmor'))
		{
			skillLevel = SkillSystem.GetSkillLevelValue(class'SkillEnviro');
			newDamage *= 0.5 * skillLevel;
		}
	}

	if(damageType == 'HalonGas')
	{
		if(bOnFire && !bCheckOnly)
			ExtinguishFire();
	}
	else if((damageType == 'Shot') || (damageType == 'AutoShot'))
	{
		if(AugmentationSystem != None)
			augLevel = AugmentationSystem.GetAugLevelValue(class'AugBallistic');

		if(augLevel >= 0.0)
        {
            prevDamage = newDamage;
			newDamage *= augLevel;
            //if(Manager.bDrainEnergyOnUsage)
            //    damageEnergyDrain += (prevDamage-newDamage)*(_ANNAAugmentationManager(AugmentationSystem).GetEnergyUsage(class'AugBallistic') / 90);
            
        }
	}
	else if(damageType == 'EMP')
	{
		if(AugmentationSystem != None)
			augLevel = AugmentationSystem.GetAugLevelValue(class'AugEMP');

		if(augLevel >= 0.0)
        {
            prevDamage = newDamage;
			newDamage *= augLevel;
            //if(Manager.bDrainEnergyOnUsage)
            //    damageEnergyDrain += (prevDamage-newDamage)*(_ANNAAugmentationManager(AugmentationSystem).GetEnergyUsage(class'AugEMP') / 90);
            
        }
	}
	else if((damageType == 'Burned') || (damageType == 'Flamed') || (damageType == 'Exploded') || (damageType == 'Shocked'))
	{
		if(AugmentationSystem != None)
			augLevel = AugmentationSystem.GetAugLevelValue(class'AugShield');

		if(augLevel >= 0.0)
        {
            prevDamage = newDamage;
			newDamage *= augLevel;
            //if(Manager.bDrainEnergyOnUsage)
            //    damageEnergyDrain += (prevDamage-newDamage)*(_ANNAAugmentationManager(AugmentationSystem).GetEnergyUsage(class'AugShield') / 90);
            
        }
	}

	if(newDamage < Damage)
	{
		if(!bCheckOnly)
		{
			pct = 1.0 - (newDamage / Float(Damage));
			SetDamagePercent(pct);
			ClientFlash(0.01, vect(0, 0, 50));
		}
		bReduced = True;
	}
	else
	{
		if(!bCheckOnly)
			SetDamagePercent(0.0);
	}

	if((damageType == 'Shot') || (damageType == 'AutoShot'))
	{
		newDamage *= CombatDifficulty;

		if((newDamage <= 1) && (Damage > 0))
			newDamage = 1;
	}
    
    EnergyDrain += damageEnergyDrain;
    EnergyDrainTotal += damageEnergyDrain;

	adjustedDamage = Int(newDamage);
	return bReduced;
}

function Landed(vector HitNormal)
{
    local vector legLocation;
	local int augLevel;
	local float augReduce, dmg, damageEnergyDrain;

	PlayLanded(Velocity.Z);

	if(Velocity.Z < -1.4 * JumpZ)
	{
		MakeNoise(-0.5 * Velocity.Z/(FMax(JumpZ, 150.0)));
		if((Velocity.Z < -700) && (ReducedDamageType != 'All'))
        {
			if(Role == ROLE_Authority)
            {
				augReduce = 0;
				if(AugmentationSystem != None)
				{
					augLevel = AugmentationSystem.GetClassLevel(class'AugSpeed');
					if(augLevel >= 0)
                    {
						augReduce = 15 * (augLevel+1);
                        /*if(Manager.bDrainEnergyOnUsage)
                        {
                            damageEnergyDrain = _ANNAAugmentationManager(AugmentationSystem).GetEnergyUsage(class'AugSpeed')/120;

                            EnergyDrain += damageEnergyDrain;
                            EnergyDrainTotal += damageEnergyDrain;
                        }*/
                    }
				}

				dmg = Max((-0.16 * (Velocity.Z + 700)) - augReduce, 0);
				legLocation = Location + vect(-1,0,-1);			// damage left leg
				TakeDamage(dmg, None, legLocation, vect(0,0,0), 'fell');

				legLocation = Location + vect(1,0,-1);			// damage right leg
				TakeDamage(dmg, None, legLocation, vect(0,0,0), 'fell');

				dmg = Max((-0.06 * (Velocity.Z + 700)) - augReduce, 0);
				legLocation = Location + vect(0,0,1);			// damage torso
				TakeDamage(dmg, None, legLocation, vect(0,0,0), 'fell');
            }
        }
	}
	else if((Level.Game != None) && (Level.Game.Difficulty > 1) && (Velocity.Z > 0.5 * JumpZ))
		MakeNoise(0.1 * Level.Game.Difficulty);

	bJustLanded = true;
}

function DoJump( optional float F )
{
	local DeusExWeapon w;
	local float scaleFactor, augLevel, damageEnergyDrain;

	if((CarriedDecoration != None) && (CarriedDecoration.Mass > 20))
		return;
	else if(bForceDuck || IsLeaning())
		return;

	if(Physics == PHYS_Walking)
	{
		if(Role == ROLE_Authority)
			PlaySound(JumpSound, SLOT_None, 1.5, true, 1200, 1.0 - 0.2*FRand());

		if((Level.Game != None) && (Level.Game.Difficulty > 0))
			MakeNoise(0.1 * Level.Game.Difficulty);

		PlayInAir();

		Velocity.Z = JumpZ;

        if(Level.NetMode != NM_Standalone)
        {
            if(AugmentationSystem == None)
                augLevel = -1.0;
            else
            {
                augLevel = AugmentationSystem.GetAugLevelValue(class'AugSpeed');
                /*if(augLevel >= 0 && Manager != None)
                {
                    if(Manager.bDrainEnergyOnUsage)
                    {
                        damageEnergyDrain = _ANNAAugmentationManager(AugmentationSystem).GetEnergyUsage(class'AugSpeed')/120;

                        EnergyDrain += damageEnergyDrain;
                        EnergyDrainTotal += damageEnergyDrain;
                    }
                }*/
            }

            w = DeusExWeapon(InHand);
            if((augLevel != -1.0) && ( w != None ) && ( w.Mass > 30.0))
            {
                scaleFactor = 1.0 - FClamp( ((w.Mass - 30.0)/55.0), 0.0, 0.5 );
                Velocity.Z *= scaleFactor;
            }
        }

		if(Base != None && Base != Level)
			Velocity.Z += Base.Velocity.Z;

		SetPhysics(PHYS_Falling);
		if(bCountJumps && (Role == ROLE_Authority))
			Inventory.OwnerJumped();
	}
}

function GenerateTotalHealth()
{
    Super.GenerateTotalHealth();

    // HealthHead / Torso / Legs aren't replicated, so we gotta make new variables and store them in there as well!
    _HealthHeadReplicated = HealthHead;
    _HealthTorsoReplicated = HealthTorso;
    _HealthLegsReplicated = (HealthLegLeft+HealthLegRight)/2;
}

function ResetPlayerToDefaults()
{
	Super.ResetPlayerToDefaults();
    
    GenerateTotalHealth();
}

exec function BehindView(Bool B)
{
	if(!bCheatsEnabled)
		return;

	if(!bAdmin && (Level.Netmode != NM_Standalone))
		return;

    Super.BehindView(B);
}

function GrantAugs(int NumAugs)
{
    local Augmentation CurrentAug, OriginalAug;
    local int PriorityIndex;
    local int AugsLeft;

    if(Role < ROLE_Authority)
        return;

    AugsLeft = NumAugs;

    for(PriorityIndex = 0; PriorityIndex < ArrayCount(AugPrefs); PriorityIndex++)
    {
        if(AugsLeft <= 0)
            return;

        if(AugPrefs[PriorityIndex] == '')
            return;

        for(CurrentAug = AugmentationSystem.FirstAug; CurrentAug != None; CurrentAug = CurrentAug.next)
        {
            OriginalAug = CurrentAug;
            if(CurrentAug.Owner != None && CurrentAug.Owner.isA('Augmentation'))
                CurrentAug = Augmentation(CurrentAug.Owner);

            if((OriginalAug.Class.Name == AugPrefs[PriorityIndex]) && (CurrentAug.bHasIt == False))
            {
                AugmentationSystem.GivePlayerAugmentation(OriginalAug.Class);
                if(CurrentAug.bHasIt)
                    CurrentAug.CurrentLevel = CurrentAug.MaxLevel;

                AugsLeft = AugsLeft - 1;
                break;
            }
            CurrentAug = OriginalAug;
        }
    }
}

exec function SetName(coerce string S)
{
    serverSetName(S);
}

function serverSetName(string S)
{
    local int _bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.SetName(self, S, _bOverride);

    if(_bOverride != BOOL_TRUE)
        Super.SetName(S); // Wait, this actually works?
}

exec function ShowInventoryWindow()
{
    // Log("ShowInventoryWindow ANNA"@RestrictInput());
	if(RestrictInput())
		return;

    serverShowInventoryWindow();
}

function serverShowInventoryWindow()
{
    local int _bOverride;
    
    // Log("serverShowInventoryWindow ANNA"@RestrictInput());
    if(Manager == None)
        Manager = findManager();

    Manager.PlayerCommand(self, "ShowInventoryWindow", _bOverride);
    if(_bOverride != BOOL_TRUE)
        Super.ShowInventoryWindow();
}

exec function ShowSkillsWindow()
{
	if(RestrictInput())
		return;

    serverShowSkillsWindow();
}

function serverShowSkillsWindow()
{
    local int _bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.PlayerCommand(self, "ShowSkillsWindow", _bOverride);
    if(_bOverride != BOOL_TRUE)
        Super.ShowSkillsWindow();
}

exec function ShowAugmentationsWindow()
{
	if(RestrictInput())
		return;

    serverShowAugmentationsWindow();
}

function serverShowAugmentationsWindow()
{
    local int _bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.PlayerCommand(self, "ShowAugmentationsWindow", _bOverride);
    if(_bOverride != BOOL_TRUE)
        Super.ShowAugmentationsWindow();
}

exec function BuySkills()
{
    local int _bOverride;

    if(Manager == None)
        Manager = findManager();

    Manager.PlayerCommand(self, "BuySkills", _bOverride);
    if(_bOverride != BOOL_TRUE)
        Super.BuySkills();
}

simulated event Destroyed()
{
	if(PlayerIsOnServer())
	{
        if(Manager == None)
            Manager = findManager();

        Manager.PlayerDestroyed(self);
    }
    Super.Destroyed();
}

// Uh, this function is clientsided?
// Also, ShowInventory etc -> EMMA "UserFunction" / "callUserFunction". UserFunction(Player, Command, bOverride)


DefaultProperties
{
    defaultMaxFrobDistance=112.00
    PlayerReplicationInfoClass=Class'EMMAPRI'
}