//================================================================================
// EMMAModule. (C) 2012 "Dei" & Nobody
//================================================================================
class EMMAModule extends Mutator;

var private EMMAModule NextModule;

/* Module Permissions */
var config bool bModuleEnabled;
var config bool bScoreKill;
var config bool bPlayerPossess;
var config bool bFindPlayerStart;
var config bool bScoreBoard;
var config bool bMenuMain;
var config bool bGameName;
var config bool bPreLogin;
var config bool bHandleDeathNotification;
var config bool bKilled;
var config bool bCheckVictoryConditions;
var config bool bChangeTeam;
var config bool bGiveInitialInventory;
var config bool bLogout;
var config bool bPlayerDestroyed;
var config bool bSetName;
var config bool bPlayerCommand;
var config bool bDropItem;

/* Module Information */
var string ModuleName;
var string ModuleAuthor;
var string ModuleVersion;

const BOOL_FALSE = 0;
const BOOL_TRUE = 1;

function PostBeginPlay()
{
    local EMMAManager Manager;

    if(bModuleEnabled)
    {
        foreach AllActors(class'EMMAManager', Manager)
        {
            if(Manager != None)
            {
                Manager.AddModule(self);
                break;
            }
        }

        Level.Game.BaseMutator.AddMutator(Self);

        Super.PostBeginPlay();
    }
}

final function AddModule(EMMAModule M)
{
    if(M.Class != Class)
    {
        if(NextModule == None)
            NextModule = M;
        else
            NextModule.AddModule(M);
    }
}

final function IdentifyModule(out int bModulesLoaded)
{
    if(bModuleEnabled)
    {
        Log(ModuleName@"("$ModuleVersion$") by"@ModuleAuthor, 'EMMA');
        bModulesLoaded = BOOL_TRUE;
    }

    if(NextModule != None)
        NextModule.IdentifyModule(bModulesLoaded);
}

final function callScoreKill(pawn Killer, pawn Other)
{
    if(bScoreKill && bModuleEnabled)
        ScoreKill(Killer, Other);

    if(NextModule != None)
        NextModule.callScoreKill(Killer, Other);
}

function ScoreKill(pawn Killer, pawn Other)
{
}

final function NavigationPoint callFindPlayerStart(Pawn Player, optional byte InTeam, optional string incomingName)
{
    local NavigationPoint resultValue;

    if(bFindPlayerStart && bModuleEnabled)
        resultValue = FindPlayerStart(Player, InTeam, incomingName);

    if(resultValue == None && NextModule != None)
        resultValue = NextModule.callFindPlayerStart(Player, InTeam, incomingName);

    return resultValue;
}

function NavigationPoint FindPlayerStart(Pawn Player, optional byte InTeam, optional string incomingName)
{
    return None;
}

final function callPreLogin(string Options, string Address, out string Error, out string FailCode, out int bOverride)
{
    if(bPreLogin && bModuleEnabled)
        PreLogin(Options, Address, Error, FailCode, bOverride);

    if(Len(Error) == 0 && Len(FailCode) == 0 && bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callPreLogin(Options, Address, Error, FailCode, bOverride);
}

function PreLogin(string Options, string Address, out string Error, out string FailCode, out int bOverride)
{
}

final function string callScoreBoard()
{
    local string resultValue;

    if(bScoreBoard && bModuleEnabled)
        resultValue = ScoreBoard();

    if(Len(resultValue) == 0)
        if(NextModule != None)
            resultValue = NextModule.callScoreBoard();

    return resultValue;
}

function string ScoreBoard()
{
    return "";
}

final function string callGameName()
{
    local string resultValue;

    if(bGameName && bModuleEnabled)
        resultValue = GameName();

    if(Len(resultValue) == 0)
        if(NextModule != None)
            resultValue = NextModule.callGameName();

    return resultValue;
}

function string GameName()
{
    return "";
}

final function string callMenuMain()
{
    local string resultValue;

    if(bMenuMain && bModuleEnabled)
        resultValue = MenuMain();

    if(Len(resultValue) == 0)
        if(NextModule != None)
            resultValue = NextModule.callMenuMain();

    return resultValue;
}

function string MenuMain()
{
    return "";
}

final function callHandleDeathNotification(Pawn killer, Pawn killee,  out int bOverride)
{
    if(bHandleDeathNotification && bModuleEnabled)
        HandleDeathNotification(killer, killee, bOverride);

    if(bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callHandleDeathNotification(killer, killee, bOverride);
}

function HandleDeathNotification(Pawn killer, Pawn killee, out int bOverride)
{
}

final function callKilled(pawn Killer, pawn Other, name damageType, out int bOverride)
{
    if(bKilled && bModuleEnabled)
        Killed(Killer, Other, damageType, bOverride);

    if(bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callKilled(Killer, Other, damageType, bOverride);
}

function Killed(pawn Killer, pawn Other, name damageType, out int bOverride)
{
}

final function bool callCheckVictoryConditions(Pawn Killer, Pawn Killee, String Method, out int bOverride)
{
    local bool resultValue;

    if(bCheckVictoryConditions && bModuleEnabled)
        resultValue = CheckVictoryConditions(Killer, Killee, Method, bOverride);

    if(bOverride == BOOL_FALSE && NextModule != None)
        resultValue = NextModule.callCheckVictoryConditions(Killer, Killee, Method, bOverride);

    return resultValue;
}

function bool CheckVictoryConditions(Pawn Killer, Pawn Killee, String Method, out int bOverride)
{
    return false;
}

final function callChangeTeam(Pawn PawnToChange, out int NewTeam, out int bOverride)
{
    local int originalTeam;
    originalTeam = NewTeam;

    if(bChangeTeam && bModuleEnabled)
        ChangeTeam(PawnToChange, NewTeam, bOverride);

    if(originalTeam == NewTeam && bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callChangeTeam(PawnToChange, NewTeam, bOverride);
}

function ChangeTeam(Pawn PawnToChange, out int NewTeam, out int bOverride)
{
}

final function callGiveInitialInventory(DeusExPlayer Player, out int bOverride)
{
    if(bGiveInitialInventory && bModuleEnabled)
        GiveInitialInventory(Player, bOverride);

    if(bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callGiveInitialInventory(Player, bOverride);
}

function GiveInitialInventory(DeusExPlayer Player, out int bOverride)
{
}

final function callPlayerPossess(DeusExPlayer Player)
{
    if(bPlayerPossess && bModuleEnabled)
        PlayerPossess(Player);

    if(NextModule != None)
        NextModule.callPlayerPossess(Player);
}

function PlayerPossess(DeusExPlayer Player)
{
}

final function callLogout(pawn Exiting)
{
    if(bLogout && bModuleEnabled)
        Logout(Exiting);

    if(NextModule != None)
        NextModule.callLogout(Exiting);
}

function Logout(pawn Exiting)
{
}

final function callPlayerDestroyed(DeusExPlayer Player)
{
    if(bPlayerDestroyed && bModuleEnabled)
        PlayerDestroyed(Player);

    if(NextModule != None)
        NextModule.callPlayerDestroyed(Player);
}

function PlayerDestroyed(DeusExPlayer Player)
{
}


final function callSetName(DeusExPlayer Player, string NewName, out int bOverride)
{
    if(bSetName && bModuleEnabled)
        SetName(Player, NewName, bOverride);

    if(bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callSetName(Player, NewName, bOverride);
}

function SetName(DeusExPlayer Player, string NewName, out int bOverride)
{
}

final function callPlayerCommand(DeusExPlayer Player, string Command, out int bOverride)
{
    if(bPlayerCommand && bModuleEnabled)
        PlayerCommand(Player, Command, bOverride);

    if(bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callPlayerCommand(Player, Command, bOverride);
}

function PlayerCommand(DeusExPlayer Player, string Command, out int bOverride)
{
}

final function callDropItem(DeusExPlayer Player, Inventory Item, out int bOverride)
{
    if(bDropItem && bModuleEnabled)
        DropItem(Player, Item, bOverride);

    if(bOverride == BOOL_FALSE && NextModule != None)
        NextModule.callDropItem(Player, Item, bOverride);
}

function DropItem(DeusExPlayer Player, Inventory Item, out int bOverride)
{
}

defaultproperties
{
    bHidden=true
    ModuleName="DEFAULT MODULE NAME"
    ModuleAuthor="EMMA"
    ModuleVersion="V1.00"
    bModuleEnabled=true
    bHandleDeathNotification=true
    bPreLogin=true
    bScoreBoard=true
    bMenuMain=true
    bGameName=true
    bScoreKill=true
    bLogout=true
    bPlayerDestroyed=true
    bPlayerPossess=true
    bFindPlayerStart=true
    bKilled=true
    bSetName=true
    bPlayerCommand=true
    bDropItem=true
    bCheckVictoryConditions=true
    bChangeTeam=true
    bGiveInitialInventory=true
}